dofile_once( "data/scripts/lib/utilities.lua" )

--verlet rope

local num_limbs = 1

local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)
local parent_id = EntityGetParent( entity_id )
local px, py = EntityGetTransform(parent_id)

local verlet_component = EntityGetFirstComponentIncludingDisabled(entity_id, "VerletPhysicsComponent")
local verlet_world_joint_component = EntityGetFirstComponentIncludingDisabled(entity_id, "VerletWorldJointComponent")

local positions = ComponentGetValue2(verlet_component, "positions")
positions[31] = px
positions[32] = py
if verlet_component then
	ComponentSetValue2(verlet_component, "positions", positions)
end
if verlet_world_joint_component then
	-- world_position is the origin of the verlet
	ComponentSetValue2(verlet_world_joint_component, "world_position", x, y)
end

--Radius constriction
function constrain(entity, x2, y2, max_dist, force)
	local chardatacomp = EntityGetFirstComponent(entity, "CharacterDataComponent")
	local projcomp = EntityGetFirstComponent(entity, "VelocityComponent")
	local physcomp = EntityGetFirstComponent(entity, "PhysicsBodyComponent") or EntityGetFirstComponent(entity, "PhysicsBody2Component") or EntityGetFirstComponent(entity, "SimplePhysicsComponent")
	local vel_x, vel_y = ComponentGetValue2(chardatacomp, "mVelocity")
	local proj_vel_x, proj_vel_y = ComponentGetValue2(projcomp, "mVelocity")
	local x1, y1 = EntityGetTransform(entity)
	if (not x1) or (not x2) then return nil end
		
	local dist = math.sqrt(((x2-x1)^2) + ((y2-y1)^2))
	local target_x = x2 + (((x1-x2)/dist)*max_dist)
	local target_y = y2 + (((y1-y2)/dist)*max_dist)
	local target_vel_x = target_x - x1
	local target_vel_y = target_y - y1

	if dist > max_dist then
		if physcomp or not proj_vel_x then
			PhysicsApplyForce(entity, target_vel_x*force, target_vel_y*force)
		end
		if vel_x then
			vel_y = vel_y - 60
			local target_mag = math.sqrt((target_vel_x^2) + (target_vel_y^2))
			local mag = math.sqrt((vel_x^2) + (vel_y^2)) / 40
			local force_x = ((target_vel_x/target_mag)*mag)
			local force_y = ((target_vel_y/target_mag)*mag)

			vel_x = ((vel_x + force_x)*1.025) + (target_vel_x*dist*0.8)
			vel_y = ((vel_y + force_y)*1.025) + (target_vel_y*dist*0.8)
			EntitySetTransform(entity, target_x, target_y)
			if ComponentGetValue2(chardatacomp, "is_on_ground") == false then vel_x = vel_x*1.16 end
			ComponentSetValue2(chardatacomp, "mVelocity", vel_x, vel_y+60)
		elseif proj_vel_x then
			ComponentSetValue2(projcomp, "mVelocity", proj_vel_x+(target_vel_x*100), proj_vel_y+(target_vel_y*100))
			EntitySetTransform(entity, target_x, target_y)
		end
	end
end

constrain(entity_id, px, py, 150, 200)
constrain(parent_id, x, y, 150, 100)

local parent_damage_model_component = EntityGetFirstComponentIncludingDisabled(parent_id, "DamageModelComponent")
local parent_health = ComponentGetValue2(parent_damage_model_component, "hp")

if GlobalsGetValue("vehicles_level2", "0") == "0" then
	--GUI
	gui = gui or GuiCreate()
	GuiStartFrame(gui)
	local screen_width, screen_height = GuiGetScreenDimensions(gui)
	local healthbar_width = (30)
	local healthbar_height = 2
	local healthbar_offset_y = screen_height * 0.213889 -48
	local hp_max = 100
	local hp = math.floor(( parent_health / 4 ) * 100) -- SET YOUR HP HEEEEEEEEEEEEEREEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
	if hp < 0 then
	  return
	end

	local percent_filled = hp / hp_max
	local inner_healthbar_width = (healthbar_width) * percent_filled
	-- Outline / Background
	GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	GuiZSetForNextWidget(gui, -100)
	GuiImage(gui, 2, (screen_width / 2) - (healthbar_width), screen_height - healthbar_offset_y - 8, "data/ui_gfx/hud/colors_health_bar_damage.png", 1, healthbar_width, healthbar_height) -- 20x20
	-- Actual inner bar
	GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	GuiZSetForNextWidget(gui, -101)
	GuiImage(gui, 3, (screen_width / 2) - (healthbar_width), screen_height - healthbar_offset_y - 8, "data/ui_gfx/hud/colors_health_bar.png", 1, inner_healthbar_width, healthbar_height) -- 20x20
	-- Health icon
	GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	GuiZSetForNextWidget(gui, -102)
	GuiImage(gui, 4, (screen_width / 2 ) + ( healthbar_width / 2 ) + 17, screen_height - healthbar_offset_y - 10, "data/ui_gfx/hud/health.png", 1, 1, 1)
	-- Text
	local text_width = GuiGetTextDimensions(gui, hp, 1)
	GuiColorSetForNextWidget(gui, 1, 1, 1, 1)
	GuiZSetForNextWidget(gui, -103)
	GuiText(gui, (screen_width / 2) - (text_width / 2), screen_height - healthbar_offset_y - 1, hp .. "%")
end