dofile_once("data/scripts/lib/utilities.lua")

local entity_id = GetUpdatedEntityID()
local x, y, rotation = EntityGetTransform(entity_id)

--half width and height
local w = 30
local h = 20

-- Define the coordinates of the four corners of the rectangle
local right_x, right_y = vec_rotate(w, -h, rotation)
right_x = x + right_x
right_y = y + right_y

local left_x, left_y = vec_rotate(-w, -h, rotation)
left_x = x + left_x
left_y = y + left_y

local right_x2, right_y2 = vec_rotate(w, h, rotation)
right_x2 = x + right_x2
right_y2 = y + right_y2

local left_x2, left_y2 = vec_rotate(-w, h, rotation)
left_x2 = x + left_x2
left_y2 = y + left_y2

--Checking if coordinate is inside the rotating rectangle
local function pointInsideRect(x2, y2)
  local nx, ny = vec_rotate(x2 - x, y2 - y, -rotation)
  return (nx + x) >= x - w and (ny + y) >= y - h and (nx + x) <= x + w and (ny + y) <= y + h
end

function getClosestPointOnLine(line_x1, line_y1, line_x2, line_y2, x, y)
  local A = x - line_x1
  local B = y - line_y1
  local C = line_x2 - line_x1
  local D = line_y2 - line_y1

  local dot = A * C + B * D
  local len_sq = C * C + D * D
  local param = dot / len_sq

  local closest_x, closest_y
  if param < 0 then
    closest_x = line_x1
    closest_y = line_y1
  elseif param > 1 then
    closest_x = line_x2
    closest_y = line_y2
  else
    closest_x = line_x1 + param * C
    closest_y = line_y1 + param * D
  end

  local dx = x - closest_x
  local dy = y - closest_y
  local distance_squared = dx * dx + dy * dy

  return closest_x, closest_y, distance_squared
end

function getClosestPointOnLines(lines, x, y)
  local closest_point, closest_line
  local closest_distance2 = math.huge
  for i, line in ipairs(lines) do
    local x, y, dist2 = getClosestPointOnLine(line[1], line[2], line[3], line[4], x, y)
    if dist2 < closest_distance2 then
      closest_distance2 = dist2
      closest_point = { x = x, y = y, dist2 = dist2 }
      closest_line = line
    end
  end
  return closest_point, closest_line
end

local particle_count = 0

for i, particle in ipairs(EntityGetAllChildren(entity_id) or {}) do

  if EntityHasTag( particle, "liquid_particle" ) == true then
	particle_count = particle_count + 1
  end
  
  local px, py = EntityGetTransform(particle)
  lines = {
    { left_x, left_y, right_x, right_y },
    { left_x, left_y, left_x2, left_y2 },
    { right_x, right_y, right_x2, right_y2 },
    { left_x2, left_y2, right_x2, right_y2 },
  }
  local closest_point, closest_line = getClosestPointOnLines(lines, px, py)
  
  if not pointInsideRect(px, py) then
    pvx = (closest_point.x - px) * 1.5
    pvy = (closest_point.y - py) * 1.5

	EntitySetTransform(particle, closest_point.x, closest_point.y)

	local physcomp = EntityGetFirstComponent(particle, "PhysicsBodyComponent") or EntityGetFirstComponent(particle, "PhysicsBody2Component") or EntityGetFirstComponent(particle, "SimplePhysicsComponent")
	if physcomp then
		PhysicsApplyForce(particle, pvx * 0.1, pvy * 0.1)
	end
    if closest_line == lines[1] then
	
		EntityLoad("mods/vehicles/files/truck/liquid_container_particle_outside.xml", closest_point.x, closest_point.y)
		EntityKill(particle)
    end
  end
end

if GlobalsGetValue("vehicles_level1", "0") == "0" then
	if GlobalsGetValue("vehicles_respawn_in_progress", "0") == "0" then
		if particle_count < 25 then

			local parent_id = EntityGetParent(entity_id)
			local parent_x, parent_y = EntityGetTransform(parent_id)
			local phrases = { "Mission Failed", "Try Again" }
			SetRandomSeed(GameGetFrameNum(), GameGetFrameNum())
			GamePrintImportant(phrases[Random(1, #phrases)], "You lost too much liquid!")
			EntitySetComponentsWithTagEnabled(parent_id, "controls", false)
			GamePlaySound( "data/audio/Desktop/event_cues.bank", "event_cues/angered_the_gods/create", parent_x, parent_y )
			GlobalsSetValue("vehicles_respawn_in_progress", "1")
			SetTimeOut(4, "mods/vehicles/files/restart.lua", "clear")
			SetTimeOut(5, "mods/vehicles/files/restart.lua", "respawn")

		end
	end

	gui = gui or GuiCreate()
	GuiStartFrame(gui)
	local screen_width, screen_height = GuiGetScreenDimensions(gui)
	local healthbar_width = (50)
	local healthbar_height = 2
	local healthbar_offset_y = screen_height * 0.213889 -48
	local hp_max = 100
	local hp = math.floor(( particle_count / 60 ) * 100) -- SET YOUR HP HEEEEEEEEEEEEEREEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
	if hp < 0 then
	  return
	end

	local percent_filled = hp / hp_max
	local inner_healthbar_width = (healthbar_width) * percent_filled
	-- Outline / Background
	GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	GuiZSetForNextWidget(gui, -100)
	GuiImage(gui, 2, (screen_width / 2) - (healthbar_width), screen_height - healthbar_offset_y - 8, "data/ui_gfx/hud/colors_health_bar_damage.png", 1, healthbar_width, healthbar_height) -- 20x20
	-- Actual inner bar
	GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	GuiZSetForNextWidget(gui, -101)
	GuiImage(gui, 3, (screen_width / 2) - (healthbar_width), screen_height - healthbar_offset_y - 8, "data/ui_gfx/hud/colors_health_bar.png", 1, inner_healthbar_width, healthbar_height) -- 20x20
	-- Treshold Mark
	GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	GuiZSetForNextWidget(gui, -101)
	GuiImage(gui, 3, ( screen_width / 2 ) + ((-healthbar_width / 2) + healthbar_width * (25/60)) - 5, screen_height - healthbar_offset_y - 8, "data/ui_gfx/hud/colors_flying_bar.png", 1, 2, healthbar_height) -- 20x20
	-- Health icon
	-- GuiOptionsAddForNextWidget(gui, 2) -- NonInteractive
	-- GuiZSetForNextWidget(gui, -102)
	-- GuiImage(gui, 4, screen_width / 2, screen_height - healthbar_offset_y - 20, "data/ui_gfx/hud/potion.png", 1, 1, 1) -- 20x20
	-- Text
	local text_width = GuiGetTextDimensions(gui, hp, 1)
	GuiColorSetForNextWidget(gui, 1, 1, 1, 1)
	GuiZSetForNextWidget(gui, -103)
	GuiText(gui, (screen_width / 2) - (text_width / 2), screen_height - healthbar_offset_y - 1, hp .. "%")
	-- "Player" string
	-- local player_string = GameTextGetTranslatedOrNot("$animal_player")
	-- local text_width = GuiGetTextDimensions(gui, player_string, 1)
	-- GuiColorSetForNextWidget(gui, 0.8, 0.8, 0.8, 1)
	-- GuiZSetForNextWidget(gui, -103)
	-- GuiText(gui, (screen_width - text_width) / 2, screen_height - healthbar_offset_y - 13, player_string)
end